package ink.wgink.module.file.controller.api.fileclient;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.file.pojo.dtos.fileclient.FileClientDTO;
import ink.wgink.module.file.pojo.vos.fileremote.FileClientVO;
import ink.wgink.module.file.service.fileclient.IFileClientService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: FileClientController
 * @Description: 文件远程
 * @Author: wanggeng
 * @Date: 2022/8/4 22:18
 * @Version: 1.0
 */
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "文件客户端接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/file-client")
public class FileClientController extends DefaultBaseController {

    @Autowired
    private IFileClientService fileRemoteService;

    @ApiOperation(value = "新增", notes = "新增接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult save(@RequestBody FileClientVO fileClientVO) {
        fileRemoteService.save(fileClientVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "删除(id列表)", notes = "删除(id列表)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@PathVariable("ids") String ids) throws RemoveException {
        fileRemoteService.remove(Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改", notes = "修改接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileClientId", value = "主键", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("update/{fileClientId}")
    @CheckRequestBodyAnnotation
    public SuccessResult update(@PathVariable("fileClientId") String fileClientId, @RequestBody FileClientVO fileClientVO) {
        fileRemoteService.update(fileClientId, fileClientVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "详情(通过ID)", notes = "详情(通过ID)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "fileClientId", value = "主键", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{fileClientId}")
    public FileClientDTO get(@PathVariable("fileClientId") String fileClientId) {
        return fileRemoteService.get(fileClientId);
    }

    @ApiOperation(value = "列表", notes = "列表接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<FileClientDTO> list() throws SearchException {
        Map<String, Object> params = requestParams();
        return fileRemoteService.list(params);
    }

    @ApiOperation(value = "分页列表", notes = "分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<FileClientDTO>> listPage(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return fileRemoteService.listPage(page);
    }

}
